from OFS.Folder import manage_addFolder, Folder
from zLOG import LOG, INFO, ERROR
from AccessControl import ClassSecurityInfo, getSecurityManager
from Globals import InitializeClass, package_home
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.PageTemplates.ZopePageTemplate import PageTemplate, manage_addPageTemplate

from App.Common import package_home
from os import path
from urllib import quote, unquote

import Products.Silva.SilvaPermissions as SilvaPermissions
from Products.Silva import roleinfo
from Products.Silva.interfaces import IVersion,IVersionedContent,IContainer
from Products.Silva.adapters import version_management

from Products.SilvaFind.interfaces import ISilvaQuery

icon = "www/wiki-tool.jpg"

class SilvaWikiToolbar(Folder):
    meta_type = "Silva Wiki Toolbar"
    title = "Silva Wiki Toolbar"

    def __init__(self, id):
        self.id = id
    manage_options = (Folder.manage_options)
    security = ClassSecurityInfo()

    #test to see if user is authenticated.
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'isAuthenticated')
    def isAuthenticated(self, user):
        from AccessControl.User import nobody
        return user!=nobody

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'getDocInfo')
    def getDocInfo(self, model):
        """This should return (doc,doc_ver) for only Versioned Content objects
          (read: silva documents)
          Content objects return (Content,None)
          Containers without versioned content as index return (Default Content,None)
          Containers without an index return (None,None)
          Anything else should be (None,None).

          Cases:
          Accessing a folder:
            If model is a container, we only want to edit it's default document
            if it is a SilvaDocument.  Nothing else (e.g. AutoTocs) are very
            interesting, so edit the container.
          Accessing a document:
            very simple: retrieve document and document version
          Accessing a document version (browsing revision history)
            1) return (document, current version)?
            2) return (current version, current version)?
            I'm opting for (1).  Since we have the version, we can render it
            in revision history, and we can compare model to the version.
            """
        if IContainer.providedBy(model):
            default = model.get_default()
            if not default:
                return (None,None)
            elif not IVersionedContent.providedBy(default):
                return (default,None)
            else:
                model = default
        if IVersionedContent.providedBy(model):
            return (model,model.get_viewable())
        elif IVersion.providedBy(model):
            return (model.object(),model)
        return (model,None)
                
    #test to see if user can access the SMI.
    #meaning: user has at least READER role.
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'canAccessSMI')
    def canAccessSMI(self, model, user):
        return user.has_role(roleinfo.READER_ROLES,model)

    #returns the documents siblings list
    #items user cannot view are filtered out.
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'getViewableItems')
    def getViewableItems(self, model):
        default_document =  model.get_default()
        default_doc_list =  default_document and [default_document] or []
        items = default_doc_list + model.get_ordered_publishables()
        user = self.REQUEST.AUTHENTICATED_USER
        viewable_items = []
        for i in items:
            if user.has_permission('View',i) and not ISilvaQuery.providedBy(i):
                viewable_items.append(i)
        return viewable_items

    #returns the author info for a particular document version.
    #used in the document revision wikipanel.
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'getWikiRevInfo')
    def getWikiRevInfo(self, doc_ver):
        #for retrieving the last author info of a
        #document version
        adapter = version_management.getVersionManagementAdapter(doc_ver.object())

        info = {}
        info['author'] = adapter.getVersionLastAuthorInfo(doc_ver.id).fullname()
        info['creator'] = adapter.getVersionCreatorInfo(doc_ver.id).fullname()
        info['published'] = adapter.getVersionPublicationTime(doc_ver.id)
        info['modified'] = adapter.getVersionModificationTime(doc_ver.id)
        info['status'] = adapter.getVersionStatus(doc_ver.id)
        return info

    #Returns html code providing links to the next and previous versions.
    #Default toolbar implementation does not display this for anonymous users.
    #(it doesn't display the revision wikipanel)
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'getWikiVersionsLink')
    def getWikiVersionsLink(self, doc, doc_ver):
        if doc.meta_type == 'Silva Document':
            model = doc
        elif doc.meta_type == 'Silva Document Version':
            doc_ver = doc
            model = doc.aq_parent
        else:
            return 'none'
    
        from Products.Silva.adapters import version_management
        adapter = version_management.getVersionManagementAdapter(model)

        versions = adapter.getVersions()
        vid = versions.index(doc_ver)
        html = []
        prev = vid > 0 and versions[vid-1] or None
        post = len(versions) > vid+1 and versions[vid+1] or None
        if prev or post:
            html.append('(')
        if prev:
            html.append('<a href="%s/show-revision">&laquo;%s</a>'%(prev.absolute_url(),prev.id))
        if post:
            if prev:
                html.append(' &ndash; ')
            html.append('<a href="%s/show-revision">%s&raquo;</a>'%(post.absolute_url(),post.id))
        if prev or post:
            html.append(')')
        return ''.join(html)

    #retrieve the root of this wiki.
    #the "wiki root" is the parent container of this toolbar.
    #just a helper funciton to get this container.
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'getWikiRoot')
    def getWikiRoot(self):
        return self.aq_inner.aq_parent

    """Plan for subscribe: only allow known users to subscribe (read: only authenticated users)
    This subscribe function can get their email address and automatically subscribe them.
    This is on the todo list"""
    #"""subscribe user"""
    #def subscribe(self):
    #    from Products.Silva import subscriptionerrors as errors
    #    request = context.REQUEST
    #    request.response.setHeader('Content-Type','text/html;charset=UTF-8')
    #    model = context
    #    service = context.service_subscriptions
    #    if not service.isSubscribable(model):
    #        return '"%s" is not subscribable.'%model.get_title()
    #    return self.absolute_url()

InitializeClass(SilvaWikiToolbar)

manage_addSilvaWikiToolbar = PageTemplateFile('www/wikiAdd',globals(),
                                                   __name__='manage_addSilvaWikiToolbar')

def addSilvaWikiToolbar(self,id, REQUEST=None):
    """ADD a toolbar to a container"""
    self._setObject(id, SilvaWikiToolbar(id))
    ob = getattr(self, id)
    fd = open(path.join(package_home(globals()),
                     'default','toolbar-macro.pt'),'r')
    manage_addPageTemplate(ob,'toolbar-macro',text=fd)
    fd.close()

    fd = open(path.join(package_home(globals()),
                     'default','web_template.html'),'r')
    manage_addPageTemplate(ob,'web_template.html',text=fd)
    fd.close()
    
    fd = open(path.join(package_home(globals()),
                     'default','wiki-styles.css.pt'),'r')
    manage_addPageTemplate(ob,'wiki-styles.css',text=fd)
    fd.close()
    fd = open(path.join(package_home(globals()),
                     'default','wiki-print.css.pt'),'r')
    manage_addPageTemplate(ob,'wiki-print.css',text=fd)
    fd.close()
    fd = open(path.join(package_home(globals()),
                     'default','show-revision.py'),'r')
    manage_addPageTemplate(ob,'show-revision.py',text=fd)
    fd.close()
    if REQUEST is None:
        return ob
    else:
        try:
            u = self.DestinationURL()
        except AttributeError:
            u = REQUEST['URL1']
        u = "%s/%s"%(u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')
